#--------------------------------------------------------------------------------
# 참조 모듈 목록.
#--------------------------------------------------------------------------------
from __future__ import annotations
from typing import Awaitable, Callable, Final, Generic, Iterable, Iterator, Optional, Sequence, Type, TypeVar, Tuple, Union
from typing import ItemsView, KeysView, ValuesView
from typing import IO, TextIO, BinaryIO
from typing import Any, List, Dict, Set
from typing import cast, overload
import builtins


#--------------------------------------------------------------------------------
# 패키지 포함 목록.
#--------------------------------------------------------------------------------
from .common import BooleanAttribute, Content, GuidAttribute, IdAttribute, WXSObject
from .data import Component, ComponentGroup, ComponentGroupRef, ComponentRef, Condition, CreateFolder, Custom, CustomAction,\
Directory, DirectoryRef, DirectorySearch, Feature, File, FileSearch, Fragment, InstallExecuteSequence, Package, Product, Property,\
UI, UIRef, Wix, WixVariable
from .wxsdocument import WXSDocument
from .builder import Builder
from .languagecodetype import LanguageCodeType


#--------------------------------------------------------------------------------
# 참조 모듈 목록.
#--------------------------------------------------------------------------------
from __future__ import annotations
from typing import Awaitable, Callable, Final, Generic, Iterable, Iterator, Optional, Sequence, Type, TypeVar, Tuple, Union
from typing import ItemsView, KeysView, ValuesView
from typing import IO, TextIO, BinaryIO
from typing import Any, List, Dict, Set
from typing import cast, overload
import builtins


#--------------------------------------------------------------------------------
# 패키지 포함 목록.
#--------------------------------------------------------------------------------
from .deprecated import SimpleWXS
from .filesystem import DirectoryNode, FileNode, Node, NodeManager, NodeType
from .wix3 import BooleanAttribute, Content, GuidAttribute, IdAttribute, WXSObject
from .wix3 import Component, ComponentGroup, ComponentGroupRef, ComponentRef, Condition, CreateFolder, Custom, CustomAction, Directory,\
					DirectoryRef, DirectorySearch, Feature, File, FileSearch, Fragment, InstallExecuteSequence, Package, Product, Property,\
					UI, UIRef, Wix, WixVariable
from .wix3 import Builder, LanguageCodeType, WXSDocument
from .commandmanager import CommandManager


#--------------------------------------------------------------------------------
# 참조 모듈 목록.
#--------------------------------------------------------------------------------
from __future__ import annotations
from typing import Awaitable, Callable, Final, Generic, Iterable, Iterator, Optional, Sequence, Type, TypeVar, Tuple, Union
from typing import ItemsView, KeysView, ValuesView
from typing import IO, TextIO, BinaryIO
from typing import Any, List, Dict, Set
from typing import cast, overload
import builtins
import re
from .data import Product
from ..xml import Document, Element, Path
from .wxsdocument import WXSDocument


#--------------------------------------------------------------------------------
# WindowsInstaller XML Schema 문서 템플릿.
#--------------------------------------------------------------------------------
class WXSDocumentTemplate:
	#--------------------------------------------------------------------------------
	# 기본.
	# - wxsDocument: WXSDocument = WXSDocument(WXSDocumentTemplate.Default())
	#--------------------------------------------------------------------------------
	@staticmethod
	def Default() -> Document:
		wix: Element = Element.Create("Wix")
		product: Element = Element.Create("Product")
		wix.AddChild(product)
		package: Element = Element.Create("Package")
		product.AddChild(package)
		property: Element = Element.Create("Property", { "Id": "WIXUI_INSTALLDIR", "Value": "AltavaMaxPluginDirectory" })
		product.AddChild(property)
		mediaTemplate: Element = Element.Create("MediaTemplate", { "EmbedCab": "yes" })
		product.AddChild(mediaTemplate)
		feature: Element = Element.Create("Feauture", { "Id": "MainComponents" })
		product.AddChild(feature)
		componentGroupRef: Element = Element.Create("ComponentGroupRef", { "Id": "MainComponents" })
		feature.AddChild(componentGroupRef)
		wixVariable: Element = Element.Create("WixVariable", { "Id": "WixUILicenseRtf", "Value": "" })
		product.AddChild(wixVariable)
		ui: Element = Element.Create("UI")
		product.AddChild(ui)
		uiRef: Element = Element.Create("UIRef", { "Id": "WixUI_InstallDir" })
		ui.AddChild(uiRef)
		fragment: Element = Element.Create("Fragment")
		wix.AddChild(fragment)
		directroy: Element = Element.Create("Directory", { "Id": "TARGETDIR", "Name": "SourceDir" })
		fragment.AddChild(directroy)
		componentGroup = Element.Create("ComponentGroup", { "Id": "MainComponents" })
		fragment.AddChild(componentGroup)
		document: Document = Document.Create(wix)
		return document


#--------------------------------------------------------------------------------
# 참조 모듈 목록.
#--------------------------------------------------------------------------------
from __future__ import annotations
from typing import Awaitable, Callable, Final, Generic, Iterable, Iterator, Optional, Sequence, Type, TypeVar, Tuple, Union
from typing import ItemsView, KeysView, ValuesView
from typing import IO, TextIO, BinaryIO
from typing import Any, List, Dict, Set
from typing import cast, overload
import builtins
from ...xml import Element
from pydantic import BaseModel, Field


#--------------------------------------------------------------------------------
# WXS 문서의 요소로 쓰이는 오브젝트.
#--------------------------------------------------------------------------------
T = TypeVar("T", bound = "WXSObject")
class WXSObject(BaseModel):
	#--------------------------------------------------------------------------------
	# 멤버 변수 목록.
	#--------------------------------------------------------------------------------
	Children: Optional[List[WXSObject]] = Field(default_factory = list)


	#--------------------------------------------------------------------------------
	# 오브젝트를 엘리먼트로 변경.
	#--------------------------------------------------------------------------------
	def CreateElementFromObject(wxsObject: WXSObject) -> Element:
		element = Element.Create(__class__.__name__, wxsObject.__dict__)
		return element
	

	#--------------------------------------------------------------------------------
	# 엘리먼트를 오브젝트로 변경.
	#--------------------------------------------------------------------------------
	@classmethod
	def CreateObjectFromElement(classType: Type[T], element: Element) -> T:
		globals: dict = builtins.globals()
		
		obj = classType()
		obj.__dict__.update(element.Attributes)

		argumentDictionary = dict(element.Attributes)
		childObjects = list()
		for childElement in element.Children:
			childClassType = globals.get(childElement.Name)
			if not childClassType:
				continue
			if not issubclass(childClassType, WXSObject):
				continue
			childObject = childClassType.CreateObjectFromElement(childElement)
			childObjects.append(childClassType.CreateObjectFromElement(childElement))
		argumentDictionary["Children"] = childObjects
		return classType(**argumentDictionary)
